% fig5and6.m
% (siso.m)
% compares some non-adaptive equalizer designs
% in terms of BER
% 
% Written by Rick Martin, 01/26/2004
% Revised from J06 to C13 on 02/09/2004
% Last updated on 02/13/2004
% Cornell University Blind Equalization Research Group
% 
% Copyright 2004 Cornell University, All Rights Reserved

%rand('state',state0); randn('state',state1);
state0 = rand('state'); state1 = randn('state');

% for Rayleigh fading channels (fig5):
%    tLw=48, nu=16, Nchans=100, and rawiter=100
% for Rayleigh fading channels (fig6):
%    tLw=8, nu=8, Nchans=100, and rawiter=100
%    (after loop 8, it starts again on loop 1
%    with a new set of 100 symbols)

Nchans = 100;          % number of channels to simulate
%Nchans = 28000;        % number of channels to simulate
chantype = 1;          % 1 = Rayleigh fading, 2 = ARMA, 3 = CSA
tLw = 48;              % TEQ length for Rayleigh fading channels
%tLw = 8;               % TEQ length for CSA loops
Lw = tLw-1;            % TEQ order
nu = 16;               % CP length for Rayleigh fading channels
%nu = 8;                % CP length for CSA loops
N = 64;                % FFT size
M = N+nu;              % total symbol size
SNRdB = [0:5:60]';     % SNR in dB
Lsnr = length(SNRdB);  % number of SNR values
tLh_mid = 32;          % delay spread of remote scatterers
tLh_local = 6;         % delay spread of each set of local scatterers
decay = 2;             % channel exponential decay rate
rawiter = 100;         % number of blocks of data to send
Kest = 100;            % number of blocks for TEQ estimation
ssx = 1;               % signal power
sx = sqrt(ssx);        % signal standard deviation
alpha = 1/2;           % signal weighting

%------------------------------------------
% generate transmitted data and noise

x = zeros(M*rawiter,1);
Xtrue = sign(randn(2*N*rawiter,1));
Xdd = zeros(size(Xtrue));
Xinit = (1+j)*ones(N,1);
for k=1:rawiter
   X = Xtrue((k-1)*(2*N)+[1:2:2*N]) + j*Xtrue((k-1)*(2*N)+[2:2:2*N]);
   x([nu+1:M]'+M*(k-1)) = sqrt(N)*ifft(X,N);
   x([1:nu]'+M*(k-1)) = x([N+1:N+nu]'+M*(k-1));  % add CP
   % differential decoding:
   temp = (angle(X) - angle(Xinit))/pi;
   Xdd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Xdd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Xinit = X;
end % for k=1:rawiter
x = x*(sx/sqrt(2));

n = (1/sqrt(2)) * (randn(M*rawiter,1)+j*randn(M*rawiter,1));

%------------------------------------------
hh = waitbar(0,'Iterating through channels and SNRs...');
BER = zeros(Lsnr,6);
for channum=1:Nchans

% generate channel

if chantype==1 % Rayleigh fading channel
   hmid = exp(-[1:tLh_mid].'/(tLh_mid/decay)) .* ...
      (randn(tLh_mid,1) + j*randn(tLh_mid,1));
   hlocal1 = randn(tLh_local,1) + j*randn(tLh_local,1);
   hlocal2 = randn(tLh_local,1) + j*randn(tLh_local,1);
   h = conv(conv(hlocal1,hmid),hlocal2);
elseif chantype==2 % ARMA channel
   p1 = 0.85;
   p2 = 0.65+0.45*j;
   p3 = 0.65-0.45*j;
   MApart = randn(10,1) + j*randn(10,1);
   ARpart = poly([p1,p2,p3]);
   imp = zeros(42,1); imp(1)=1;
   h = filter(MApart,ARpart,imp);
elseif chantype==3 % CSA loops
   channum2 = mod(channum-1,8)+1;
   eval(['load csaloop',num2str(channum2),'.time']);
   eval(['h = csaloop',num2str(channum2),'(:,2);']);
   h = h(1:4:end/2);
end
h = h/norm(h);
tLh = length(h);  % channel length
tLc = tLh+tLw-1;  % length of effective channel
dmin = 0;         % minimum allowable delay
dmax = tLc-1-nu;  % maximum allowable delay

for SNRi = 1:Lsnr %---------------

SNR = SNRdB(SNRi);   % SNR in dB
g = 10^(-SNR/10);    % 1/SNR, linear scale
ssn = g*ssx;         % noise power
sn = sqrt(ssn);

% generate received data
r = conv(h,x);
r = r(1:length(n)) + sn*n;

% compute various TEQs and equalize

% 1: no TEQ -----------------------------------

% compute optimal delay of channel
d1 = MaxWindow(h,nu+1);

y = r;

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d1),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,1) = BER(SNRi,1) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% 2: MMSE, assuming white signal and noise ----

% compute optimal delay and MMSE TEQ
[w2,d2] = mmse(h,nu+1,tLw,dmin,dmax,ssx,ssn);

y = conv(r,w2);

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d2),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,2) = BER(SNRi,2) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% 3: MSSNR ------------------------------------

% compute optimal delay and MSSNR TEQ
[w3,d3] = mssnr(h,nu+1,tLw,dmin,dmax);

y = conv(r,w3);

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d3),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,3) = BER(SNRi,3) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% 4: MDS-Q --------------------------------

% compute optimal delay and MDS-Q TEQ
nmin = 0;      % smallest centroid location
nmax = tLc-1;  % largest centroid location
LorQ = 1;      % quadratic weights
[w4,d4,dummy] = mdsteq(h,tLw,nmin,nmax,LorQ,alpha,ssx,ssn);

y = conv(r,w4);

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d4),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,4) = BER(SNRi,4) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% 5: MDS-L ------------------------------------

% compute optimal delay and MDS-L TEQ
nmin = 0;      % smallest centroid location
nmax = tLc-1;  % largest centroid location
LorQ = 0;      % linear weights
[w5,d5,dummy] = mdsteq(h,tLw,nmin,nmax,LorQ,alpha,ssx,ssn);

y = conv(r,w5);

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d5),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,5) = BER(SNRi,5) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% 6: Min-IBI ------------------------------------

% compute optimal delay and MDS-L TEQ
LorQ = 0;      % linear weights
[w6,d6] = minibi(h,tLw,nu,dmin,dmax,LorQ,alpha,ssx,ssn);

y = conv(r,w6);

Xest = zeros(size(Xtrue));
Ydd = zeros(size(Xtrue));
Yinit = (1+j)*ones(N,1);
for k=1:rawiter-1
   Y = fft(y((k-1)*M+[nu+1:M]+d6),N)*sqrt(2)/sqrt(N);
   Xest((k-1)*(2*N)+[1:2:2*N]) = sign(real(Y));
   Xest((k-1)*(2*N)+[2:2:2*N]) = sign(imag(Y));
   % differential decoding:
   temp = (angle(Y) - angle(Yinit))/pi;
   Ydd((k-1)*(2*N)+[1:2:2*N]) = sign(mod(temp+0.25,2)-1);
   Ydd((k-1)*(2*N)+[2:2:2*N]) = sign(mod(temp-0.25,2)-1);
   Yinit = Y;
end % for k=1:rawiter-1

BER(SNRi,6) = BER(SNRi,6) + sum(Ydd(2*N+1:2*(N-1)*rawiter) ...
   ~= Xdd(2*N+1:2*(N-1)*rawiter)) / (2*(N-2)*rawiter);

% ---------------------------------------------

end % for SNRi = 1:Lsnr
waitbar(channum/Nchans,hh);
end % for channum=1:Nchans
close(hh);
%------------------------------------------

BER = BER/Nchans;

figure(1); clf;
set(0,'DefaultAxesFontSize',16);
semilogy(SNRdB,BER(:,1),'bo-',...
     SNRdB,BER(:,2),'rs-',...
     SNRdB,BER(:,3),'gx-',...
     SNRdB,BER(:,4),'kd-',...
     SNRdB,BER(:,5),'kd--',...
     SNRdB,BER(:,6),'m+-.');
axis([SNRdB(1) SNRdB(end) 1e-4 0.5]);
%title('BER vs. SNR','FontSize',16);
xlabel('signal-to-noise ratio (SNR), in dB','FontSize',16);
ylabel('bit error rate (BER)','FontSize',16);
legend('No TEQ','MMSE','MSSNR','MDS-Q','MDS-L','Min-IBI',3);
grid on;
