function [wopt,dopt] = mmse(h,Nb,Nw,Dmin,Dmax,ssx,ssn)
% [wopt,dopt] = mmse(h,Nb,Nw,Dmin,Dmax,ssx,ssn)
%
%    h = channel
%    Nb = window length = nu+1, usually
%    Nw = TEQ length
%    [Dmin,Dmax] = range of delays to check
%       delays start at 0, not 1
%    ssx = signal power
%    ssn = noise power

% intialize variables
h = h(:);
lambdaopt = 0;
H = convmtx(h,Nw);
C = H' * H + (ssn/ssx)*eye(Nw);

for delay = Dmin:Dmax % for each delay to be searched
   Hwin = H(delay+1:delay+Nb,:);
   B = Hwin'*Hwin;
   [evecs,evals] = eig(B,C);
   [lambda,ind] = max(diag(evals));
   if lambda > lambdaopt
      % save the TEQ, delay, and energy
      wopt = evecs(:,ind);
      dopt = delay;
      lambdaopt = lambda;
   end
end

% normalize the energy of the effective channel
wopt = wopt ./ sqrt(wopt.' * C * wopt);
